#include "dumputils.hpp"

#include <iostream>
#include <components/esm/aisequence.hpp>

#include "../mwbase/world.hpp"
#include "../mwbase/environment.hpp"

#include "../mwworld/class.hpp"
#include "../mwworld/cellstore.hpp"
#include "../mwworld/esmstore.hpp"

#include "steering.hpp"
#include "movement.hpp"
#include "creaturestats.hpp"

namespace MWMechanics
{
// static variables
    bool DumpUtils::gridDumped = false;



    bool DumpUtils::isWantedActor(const MWWorld::Ptr& actor, const std::string& wantedActorName)
    {
        return actor.getCellRef().getRefId() == wantedActorName;
    }

    void DumpUtils::dumpPathGrid(const MWWorld::Ptr& actor, const std::string& wantedActorName)
    {
        if (isWantedActor(actor, wantedActorName) && !gridDumped)
        {
            gridDumped = true;
            const MWBase::Environment& envirnoment = MWBase::Environment::get();
            const MWWorld::ESMStore& store = envirnoment.getWorld()->getStore();
            const ESM::Pathgrid * pathGrid = store.get<ESM::Pathgrid>().search(*getActorsCell(actor));
            dumpPathGrid(pathGrid);
        }
    }

    const ESM::Cell* DumpUtils::getActorsCell(const MWWorld::Ptr& actor)
    {
        return actor.getCell()->getCell();
    }

    void DumpUtils::dumpPathGrid(const ESM::Pathgrid * pathGrid)
    {
        if (pathGrid)
        {
            // dump nodes
            for (std::vector<ESM::Pathgrid::Point>::const_iterator it = pathGrid->mPoints.begin(); it != pathGrid->mPoints.end(); ++it)
            {
                std::cout << "[ \"Node\", " << it->mX << ", " << it->mY << "]," << std::endl;
            }

            // dump edges
            for (std::vector<ESM::Pathgrid::Edge>::const_iterator it = pathGrid->mEdges.begin(); it != pathGrid->mEdges.end(); ++it)
            {
                if (it->mV0 < it->mV1)
                {
                    std::cout << "[ \"Edge\", " << it->mV0 << ", " << it->mV1 << "]," << std::endl;
                }
            }
        }
    }

    void DumpUtils::dumpActorPosition(const MWWorld::Ptr& actor, const PathFinder& mPathFinder, const std::string& wantedActorName)
    {
        if (isWantedActor(actor, wantedActorName))
        {
            const ESM::Cell* cell = DumpUtils::getActorsCell(actor);

            osg::Vec3f pos = actor.getRefData().getPosition().asVec3();

            DumpUtils::ToCellCoordinates(pos, cell);

            std::cout << "[ \"Actor\", " << pos[0] << ", " << pos[1];
            if (0 < mPathFinder.getPathSize())
            {
                std::list<ESM::Pathgrid::Point>::const_iterator p = mPathFinder.getPath().begin();
                std::cout << ", " << p->mX << ", " << p->mY;
            }
            std::cout << "]," << std::endl;
        }

    }

    void DumpUtils::ToCellCoordinates(osg::Vec3f& pos, const ESM::Cell * cell)
    {
        if (cell->isExterior())
        {
            pos[0] -= cell->mData.mX * ESM::Land::REAL_SIZE;
            pos[1] -= cell->mData.mY * ESM::Land::REAL_SIZE;
        }
    }

}

